#include "LPC177x_8x.h"
#include "type.h"
#include "spifi_rom_api.h"
#define SPIFIROMD_PRESENT
#include "rom_drivers.h"

#define ROM_DRIVERS_PTR ((ROM *)(*((unsigned int *)0x1FFF1FF8)))

/* Allocated secondary IRAM as source address for SPIFI ROM API test. */ 
uint8_t *src_addr = (uint8_t *)LPC_PERI_RAM_BASE;
uint8_t *dest_addr;
	
const SPIFI_RTNS *spifi;
SPIFIobj obj;
SPIFIopers opers;

typedef void (*ENTRY_FUNC)(void);
static ENTRY_FUNC entry_func;

const uint8_t spifi_blinky[] = {
0x10, 0xB5, 0x0F, 0xE0, 0x00, 0x24, 0x06, 0xE0, 0x01, 0x20, 0xA0, 0x40, 0x06, 0x49, 0x88, 0x61,
0x07, 0xF0, 0xF6, 0xFF, 0x64, 0x1C, 0x04, 0x2C, 0xF6, 0xD3, 0x0F, 0x20, 0x02, 0x49, 0xC8, 0x61,
0x07, 0xF0, 0xEE, 0xFF, 0xEE, 0xE7, 0x00, 0x00, 0x00, 0x80, 0x09, 0x20
};

const uint8_t spifi_delay[] = {
0x02, 0x48, 0x00, 0xE0, 0x40, 0x1E, 0x00, 0x28, 0xFC, 0xD1, 0x70, 0x47, 0x40, 0x42, 0x0F, 0x00
};



void spifi_io_clk_init (void) {	
	LPC_SC->PCONP |= 0x00010000;

	LPC_IOCON->P2_7 &= ~0x07;
	LPC_IOCON->P2_7 |= 0x05;	/* SPIFI_CSN @ P2.7 */
	LPC_IOCON->P0_22 &= ~0x07;
	LPC_IOCON->P0_22 |= 0x05;	/* SPIFI_CLK @ P0.22 */
	LPC_IOCON->P0_15 &= ~0x07;
	LPC_IOCON->P0_15 |= 0x5;	/* SPIFI_IO2 @ P0.15 */
	LPC_IOCON->P0_16 &= ~0x07;
	LPC_IOCON->P0_16 |= 0x5;	/* SPIFI_IO3 @ P0.16 */
	LPC_IOCON->P0_17 &= ~0x07;
	LPC_IOCON->P0_17 |= 0x5;	/* SPIFI_IO1 @ P0.17 */
	LPC_IOCON->P0_18 &= ~0x07;
	LPC_IOCON->P0_18 |= 0x5;	/* SPIFI_IO0 @ P0.18 */

	return;
}

/* Erase, program, and verify the whole SPIFI memory area */
void test_spifi_program (void) {
	int i, j, rc;
		
	/* spifi_program() will call erase first. */
	for ( j = 0; j < MEM_AREA_SIZE; j+=PROG_SIZE )
	{
		opers.dest = (char *)(SPIFI_MEM_BASE + j);
		opers.length = PROG_SIZE;
		opers.scratch = NULL;
		opers.protect = 0;
		opers.options = S_FORCE_ERASE;
		rc = spifi->spifi_program (&obj, (char *)src_addr, &opers);
		if (rc) {
			while ( 1 );
		}
		dest_addr = (uint8_t *)(SPIFI_MEM_BASE + j);
		for ( i = 0; i < PROG_SIZE; i++ )
		{
			if ( src_addr[i] != *dest_addr++ )
			{
				while ( 1 );
			}
		}
	}
}

/* test cmd mode by programming a blinky program to two regions, one
at 0x8000, the other at 0x10000, once the programming is done, a call to
the cmd mode will execute code from SPIFI directly. */ 
void test_spifi_cmdmode (void) {
	int rc;
	uint32_t value[PROG_SIZE];
	uint32_t i;

	/* The blink code is to make P0.0~0.3 to blink, the toggle routine
	resides at 0x8000, the delay routine between toggle resides at 0x10000. */
	LPC_GPIO0->DIR = 0x0000000F;		/* P0.xx defined as Outputs */
	LPC_GPIO0->CLR = 0x0000000F;		/* turn off all the LEDs */

#if 0
	/* Erase two regions 0x8000 and 0x10000, one for delay, one for blinky. 
	These two erasures are not necessary as spifi_program() will call 
	spifi_erase() internally. The test added here is to validate the ROM 
	APIs. */
	opers.dest = (char *)0x8000;
	opers.length = PROG_SIZE;
	opers.scratch = NULL;
	opers.protect = 0;
	opers.options = S_VERIFY_ERASE;
	rc = spifi->spifi_erase (&obj, &opers);
	if (rc) {
		while ( 1 );
	}

	opers.dest = (char *)0x10000;
	opers.length = PROG_SIZE;
	opers.scratch = NULL;
	opers.protect = 0;
	opers.options = S_VERIFY_ERASE;
	rc = spifi->spifi_erase (&obj, &opers);
	if (rc) {
		while ( 1 );
	}
#endif
	
	/* Program two regions 0x8000 and 0x10000, one for delay, one for blinky. */	
	for (i = 0; i < PROG_SIZE; i++)
	{
	  value[i] = 0xFFFFFFFF;
	}

	// put code up at 0x28008000
	for (i = 0; i < 0xB; i++)
	{
	  value[i] = *(uint32_t *)&spifi_blinky[i * 4];
	}
	opers.dest = (char *)0x8000;   // 0x28008000
	opers.length = PROG_SIZE;
	opers.scratch = NULL;
	opers.protect = 0;
	opers.options = S_FORCE_ERASE;
	rc = spifi->spifi_program (&obj, (char *)value, &opers);
	if (rc) {
		while ( 1 );
	}
	
	for (i = 0; i < PROG_SIZE; i++)
	{
	  value[i] = 0xFFFFFFFF;
	}

	// put code up at 0x28010000
	for (i = 0; i < 0x4; i++)
	{
	  value[i] = *(uint32_t *)&spifi_delay[i * 4];
	}
	opers.dest = (char *)0x10000;  // 0x28010000
	opers.length = PROG_SIZE;
	opers.scratch = NULL;
	opers.protect = 0;
	opers.options = S_FORCE_ERASE;
#if TEST_MID_LEVEL_API
	rc = spifi_program (&obj, (char *)value, &opers);
#else
	rc = spifi->spifi_program (&obj, (char *)value, &opers);
#endif
	if (rc) {
		while ( 1 );
	}

	/* Set to SPIFI cmd mode and execute blinky code. */
	spifi->cancel_mem_mode (&obj);
	entry_func = (ENTRY_FUNC)(SPIFI_MEM_BASE + 0x8001);
	
	// jump to SPIFI region and execute
	entry_func();
	return;
}

/******************************************************************************
** Function name:		main
**
** Descriptions:		main
**
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
int main (void) 
{
	int i, rc;

// For first silicon, ROM access is restricted to core clock of 96MHz max. 
// Therefore the SPIFI is running at half the CPU @48MHz max

  /* Stop PLL0 */
  uint32_t tmp = LPC_SC->CCLKSEL;
  LPC_SC->CCLKSEL  &= ~0x100;  /* Stop using PLL0 as input to CPU clock divider */
  LPC_SC->PLL0CON  &= ~0x01;   /* PLL0 Disable                                  */
  LPC_SC->PLL0FEED  = 0xAA;
  LPC_SC->PLL0FEED  = 0x55;

  /* Change clock frequency as max is 96MHz for LPC4088 */
  LPC_SC->PLL0CFG   = 0x00000007;
  LPC_SC->PLL0CON   = 0x01;             /* PLL0 Enable                        */
  LPC_SC->PLL0FEED  = 0xAA;
  LPC_SC->PLL0FEED  = 0x55;
  while (!(LPC_SC->PLL0STAT & (1<<10)));/* Wait for PLOCK0                    */

  LPC_SC->CCLKSEL   = tmp;   /* Go back to using PLL0 as input to CPU clock divider */
	
	/* get SPIFI API table pointer */
  spifi = ROM_DRIVERS_PTR->pSPIFID; 
	
	/* setup clocks etc */    	  
	//SystemInit();

	/* clear buffer */
	for ( i = 0; i < PROG_SIZE; i++ )
	{
		src_addr[i] = i;
	}

	/* put the clock out on two pins */	
	LPC_IOCON->P1_25 |= 5; /* CLKOUT @ P1.25 */
	LPC_IOCON->P1_27 |= 4; /* CLKOUT @ P1.27 */

	/* setup the SPIFI pins */
	spifi_io_clk_init();

	/* SPIFI clock is 8*12 = 96MHz /2 = 48Mhz */
	rc = spifi->spifi_init (&obj, 4, S_FULLCLK+S_RCVCLK, 48);
	if (rc) {
		while ( 1 ); // error
	}

	test_spifi_program();
	
	test_spifi_cmdmode();    /* Never leave here as the */	
	
	
	while ( 1 );			/* Never exit for main for easy debugging */
}
