

/*
**************************************************************************************************************
*                                       INCLUDE HEADER FILES
**************************************************************************************************************
*/

#include  "usbhost_main.h"

#define ISP1301_I2C_ADDR 0x2D
#define I2C_START_BIT (1 << 8)
#define I2C_STOP_BIT  (1 << 9)

/* i2c_txrx register definitions */
#define I2C_START    (1 << 8)		/* generate a START before this B*/
#define I2C_STOP     (1 << 9)		/* generate a STOP after this B */

/* i2c_stat register definitions */
#define I2C_TDI     (1 << 0)         /* Transaction Done Interrupt */

/* i2c_ctrl register definitions */
#define I2C_RESET   (1 << 8)         /* Soft Reset */

/* otg ctl registers */
#define OTG_CLK_I2C_EN (1 << 2)         /* Enable I2C clock */

#define I2C_READ      0x01
#define I2C_WRITE     0x00
#define DUMMY_BYTE    0x55

/* ISP1301 registers */
#define ISP1301_I2C_MODE_CONTROL_1 0x4
#define ISP1301_I2C_MODE_CONTROL_2 0x12
#define ISP1301_I2C_OTG_CONTROL_1 0x6
#define ISP1301_I2C_OTG_CONTROL_2 0x10
#define ISP1301_I2C_INTERRUPT_SOURCE 0x8
#define ISP1301_I2C_INTERRUPT_LATCH 0xA
#define ISP1301_I2C_INTERRUPT_FALLING 0xC
#define ISP1301_I2C_INTERRUPT_RISING 0xE
#define ISP1301_I2C_REG_CLEAR_ADDR 1


static int i2c_wait_reset(int timeout)
{
  volatile int i = 0;
	while(timeout > 0 && (LPC_USB->I2C_CTL & I2C_RESET))
	{
		for(i = 0; i < 0x20000; i++);
		timeout--;
	}

	return (timeout <= 0);	
}


static int isp1301_get_value(int reg)
{
	int n = 0;
  uint32_t sts = 0;

	/* send isp1301 address */
	LPC_USB->I2C_TX = ((ISP1301_I2C_ADDR << 1) | I2C_START_BIT);   

sts=LPC_USB->I2C_STS;

	/* register (offset) to read from */
	LPC_USB->I2C_TX = reg;   

sts=LPC_USB->I2C_STS;

	/* repeat start */
	LPC_USB->I2C_TX = ((ISP1301_I2C_ADDR << 1) | I2C_START_BIT | I2C_READ);
	LPC_USB->I2C_TX = (I2C_STOP_BIT | DUMMY_BYTE);

sts=LPC_USB->I2C_STS;

  sts=LPC_USB->I2C_STS;
	while ((((sts=LPC_USB->I2C_STS) & I2C_TDI) != I2C_TDI) && n++ < 100000);

	if (n>= 100000)
	{
//		printf("isp1301_get_value: ERROR TDI not set\n");
	}

	/* clear TDI */
	//LPC_USB->I2C_STS |= I2C_TDI;

	return (LPC_USB->I2C_RX & 0xff);
}

static int isp1301_set_value(int reg, int value)
{
	int n = 0;

	/* send isp1301 address */
	LPC_USB->I2C_TX = ((ISP1301_I2C_ADDR << 1) | I2C_START_BIT);   

	/* offset to write to */
	LPC_USB->I2C_TX = (reg | I2C_WRITE);   
	/* value to write */
	LPC_USB->I2C_TX = (value | I2C_STOP_BIT);  
  
	/* wait for transmit done (TDI) */
	while (((LPC_USB->I2C_STS & I2C_TDI) != I2C_TDI) && n++ < 100000);

	if (n>= 100000)
	{
//		printf("isp1301_set_value: ERROR TDI not set\n");
		return -1;		
	}
  
	/* clear TDI */
//	LPC_USB->I2C_STS |= I2C_TDI;     

	return 0;
}


int isp1301_configure()
{
  uint16_t vendId = 0;
  uint16_t prodId = 0;
  uint16_t versId = 0;

//	LPC_IOCON->P1_27 &= ~0x1F; /* USB_INT1 @ P1.27 */
//	LPC_IOCON->P1_27 |= 0x01;  /* USB_INT1 @ P1.27 */
	LPC_IOCON->P1_28 &= ~0x1F; /* USB_SCL1 @ P1.28 */
	LPC_IOCON->P1_28 |= 0x01;  /* USB_SCL1 @ P1.28 */
	LPC_IOCON->P1_29 &= ~0x1F; /* USB_SDA1 @ P1.29 */
	LPC_IOCON->P1_29 |= 0x01;  /* USB_SDA1 @ P1.29 */

	LPC_USB->I2C_CLKHI = 0xB9;
	LPC_USB->I2C_CLKLO = 0xB9; 

	LPC_USB->I2C_CTL |= I2C_RESET;	
	i2c_wait_reset(100);

//  vendId = (isp1301_get_value(0x00) | (isp1301_get_value(0x01) << 8));

  /* Mode Ctrl 2, bit 6, PSW_OE, and bit 2, BI_DI, are set */
  isp1301_set_value( ISP1301_I2C_MODE_CONTROL_2, 0x0044 );

  /* Mode Ctrl 1, bit 2, DAT_SE0, is set */
  isp1301_set_value( ISP1301_I2C_MODE_CONTROL_1, 0x04 );


  isp1301_set_value( ISP1301_I2C_OTG_CONTROL_1, 0x0021 );

  vendId = (isp1301_get_value(0x00) | (isp1301_get_value(0x01) << 8));
  prodId = (isp1301_get_value(0x02) | (isp1301_get_value(0x03) << 8));
  versId = (isp1301_get_value(0x14) | (isp1301_get_value(0x15) << 8));


  return 0;
}

